@@ -22,6 +22,7 @@
 /**
  * SECTION: gnome-scan-dialog
  * @short_description: Scan configuration dialog
+ * @include: gnome-scan.h
  *
  * The goal of the #GnomeScanDialog is to allow the user to configure
  * a #GnomeScanJob, and trigger the execution of this job by running a
@@ -34,6 +35,7 @@
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <glib/gi18n.h>
+#include "gnome-scan-private.h"
 #include "gnome-scan-init.h"
 #include "gnome-scan-dialog.h"
 #include "gnome-scan-acquisition-dialog.h"
@@ -81,7 +83,6 @@ struct _GnomeScanDialogPrivate
 	gboolean		run;
 	
 	/* 	Widgets  */
-	GtkTooltips*		tooltips;
 	GtkWidget*		notebook;
 	
 	/* General */
@@ -255,9 +256,6 @@ gnome_scan_dialog_constructor (GType type, guint n, GObjectConstructParam *param
 						   GS_STOCK_SCAN,
 						   GTK_RESPONSE_APPLY);
 	
-	priv->tooltips = gtk_tooltips_new();
-	gtk_tooltips_enable (priv->tooltips);
-	
 	/* 	Notebook  */
 	priv->notebook = gtk_notebook_new ();
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (widget)->vbox), priv->notebook,
@@ -293,6 +291,7 @@ gnome_scan_dialog_constructor (GType type, guint n, GObjectConstructParam *param
 	/* 	Processing  */
 	label = gtk_label_new_with_mnemonic (_("P_rocessing"));
 	priv->processing_box = gtk_vbox_new (FALSE, 6);
+	gtk_container_set_border_width (GTK_CONTAINER (priv->processing_box), 6);
 	priv->processing_page = priv->processing_box;
 	gtk_notebook_insert_page (GTK_NOTEBOOK (priv->notebook),
 							  priv->processing_page, label,
@@ -474,6 +473,7 @@ gnome_scan_dialog_new (GtkWindow *parent, GnomeScanJob *job)
 									"window-position", GTK_WIN_POS_CENTER,
 									"default-width", 320,
 									"default-height", 420,
+									/* translator: this is the dialog title */
 									"title", _("Scan"),
 									"has-separator", FALSE,
 									"modal", TRUE,
@@ -512,14 +512,14 @@ gnome_scan_dialog_run (GnomeScanDialog *dialog)
 	
 	if (g_slist_length (priv->backends) == 0) {
 		gsd_message_dialog (dialog, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
-							_("Unable to detect scanners !"),
+							_("Unable to detect scanners!"),
 							_("No drivers has been found."));
 		return;
 	}
 	
 	if (priv->probe_done && !priv->scanner_count) {
 		gsd_message_dialog (dialog, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
-							_("No device found !"),
+							_("No device found!"),
 							_("Ensure your device is plugged, powered, supported and configured. "
 							  "The dialog will run until you plug a scanner or cancel."));
 	}
@@ -532,6 +532,7 @@ gnome_scan_dialog_run (GnomeScanDialog *dialog)
 		g_object_get (dialog, "transient-for", &parent, NULL);
 		acquisition_dialog = gnome_scan_acquisition_dialog_new (parent, priv->job);
 		gnome_scan_acquisition_dialog_run (GNOME_SCAN_ACQUISITION_DIALOG (acquisition_dialog));
+		gtk_widget_destroy(acquisition_dialog);
 	}
 	
 	return;
@@ -629,18 +630,9 @@ gsd_build_group_box (GnomeScanDialog *dialog, GtkWidget *page, GtkBox *box, Gnom
 		shows_label = gnome_scan_param_widget_shows_label (GNOME_SCAN_PARAM_WIDGET (widget));
 		box_expands = box_expands || expands;
 		
-		/* if widget does not have it's own GdkWindow, pack it in a GtkEventBox
-		 in order to enable tooltips */
-		if (GTK_WIDGET_NO_WINDOW (widget)) {
-			eventbox = gtk_event_box_new ();
-			gtk_container_add (GTK_CONTAINER (eventbox), widget);
-			widget = eventbox;
-		}
-		
-		gtk_tooltips_set_tip (priv->tooltips, widget,
-							  dgettext (gs_param_spec_get_domain (node->data),
-										g_param_spec_get_blurb (node->data)),
-							  NULL);
+		gtk_widget_set_tooltip_text (widget,
+									 dgettext (gs_param_spec_get_domain (node->data),
+											   g_param_spec_get_blurb (node->data)));
 		
 		gtk_widget_set_no_show_all (widget, FALSE);
 		gsd_add_to_box_show (group_box, 1);
@@ -659,20 +651,14 @@ gsd_build_group_box (GnomeScanDialog *dialog, GtkWidget *page, GtkBox *box, Gnom
 			label = NULL;
 		}
 		else {
-			label = gtk_label_new (g_strdup_printf (_("%s:"),
+			label = gtk_label_new (g_strdup_printf ("%s:",
 													dgettext (gs_param_spec_get_domain (node->data),
 															  g_param_spec_get_nick (node->data))));
 			gtk_misc_set_alignment (GTK_MISC (label), 0., .5);
 			
-			/* pack label in an eventbox */
-			eventbox = gtk_event_box_new ();
-			gtk_container_add (GTK_CONTAINER (eventbox), label);
-			label = eventbox; /* substitute eventbox to label in the following code */
-			
-			gtk_tooltips_set_tip (priv->tooltips, label,
-								  dgettext (gs_param_spec_get_domain (node->data),
-											g_param_spec_get_blurb (node->data)),
-								  NULL);
+			gtk_widget_set_tooltip_text (label,
+										 dgettext (gs_param_spec_get_domain (node->data),
+												   g_param_spec_get_blurb (node->data)));
 			gtk_table_attach (GTK_TABLE (table), label,
 							  0, 1,
 							  count, count+1,
@@ -813,6 +799,7 @@ gsd_load_backends (GnomeScanDialog *dialog)
 	GThread *thread;
 	GError *error = NULL;
 	GType *backend_types;
+	GdkCursor *watch_cursor = gdk_cursor_new(GDK_WATCH);
 	gint i;
 	guint n;
 	backend_types = g_type_children (GNOME_TYPE_SCAN_BACKEND, &n);
@@ -833,11 +820,13 @@ gsd_load_backends (GnomeScanDialog *dialog)
 			
 			priv->probing_backends++;
 			
-			
+			gdk_window_set_cursor(GTK_WIDGET(dialog)->window,
+								  watch_cursor);
 			thread = g_thread_create ((GThreadFunc) gnome_scan_backend_probe_scanners,
 									  backend, FALSE, &error);
 		}
 	}
+	gdk_cursor_unref (watch_cursor);
 }
 
 static void
@@ -894,12 +883,21 @@ static void
 gsd_probe_done (GnomeScanBackend *backend, GnomeScanDialog *dialog)
 {
 	GnomeScanDialogPrivate *priv = GET_PRIVATE (dialog);
+	GdkCursor *arrow_cursor;
+	
 	priv->probe_done = !--priv->probing_backends;
 	if (!priv->scanner_count && GTK_WIDGET_VISIBLE (GTK_WIDGET (dialog))) {
 		gsd_message_dialog (dialog, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
 							_("No device found !"),
 							_("Ensure your device is plugged, powered, supported and configured."));
 	}
+
+	if (priv->probe_done) {
+		arrow_cursor = gdk_cursor_new(GDK_ARROW);
+		gdk_window_set_cursor (GTK_WIDGET(dialog)->window,
+							   arrow_cursor);
+		gdk_cursor_unref (arrow_cursor);
+	}
 }
 
 /* GENERAL */
@@ -1040,6 +1038,7 @@ gsd_build_sink_ui (GnomeScanDialog *gsd)
 	
 	groups = gnome_scan_plugin_params_get_other_groups (GNOME_SCAN_PLUGIN (sink),
 														GS_PARAM_GROUP_SINK_FRONT,
+														GS_PARAM_GROUP_HIDDEN,
 														0);
 	for (node = groups; node ; node = node->next) {
 		gsd_build_group_box (gsd, priv->sink_page,
@@ -1129,6 +1128,7 @@ gsd_build_scanner_ui	(GnomeScanDialog *gsd)
 														GS_PARAM_GROUP_SCANNER_FRONT,
 														GS_PARAM_GROUP_PREVIEW,
 														GS_PARAM_GROUP_FORMAT,
+														GS_PARAM_GROUP_HIDDEN,
 														0);
 	
 	/* build each group in the advanced page */
@@ -1169,7 +1169,9 @@ gsd_build_processing_ui(GnomeScanDialog *gsd)
 	/* loop each processor and add options */
 	for (; proc; proc = proc->next) {
 		groups = gnome_scan_plugin_params_get_other_groups (GNOME_SCAN_PLUGIN (proc->data),
-															GS_PARAM_GROUP_PREVIEW, -1);
+															GS_PARAM_GROUP_PREVIEW,
+															GS_PARAM_GROUP_HIDDEN,
+															0);
 		for (node = groups; node ; node = node->next) {
 			gsd_build_group_box (gsd, priv->processing_page,
 								 GTK_BOX (priv->processing_box),
@@ -1261,9 +1263,8 @@ gsd_preview_end_refresh (GnomeScanDialog *gsd)
 	GnomeScanSettings *settings = gnome_scan_job_get_settings (priv->preview_job);
 	
 	/* first, restore settings */
+	gnome_scan_settings_set_boolean (settings, "preview", FALSE);
 	gnome_scan_settings_set (settings, "resolution", priv->saved_res);
-	gnome_scan_settings_set (settings, "origin", priv->saved_origin);
-	gnome_scan_settings_set (settings, "paper-size", priv->saved_paper_size);
 	
 	/* restore UI */
 	GTK_WIDGET_SET_VISIBLE (priv->preview_acquisition_box, FALSE);
@@ -1275,7 +1276,7 @@ gsd_preview_end_refresh (GnomeScanDialog *gsd)
 		g_warning (G_STRLOC ": preview failed");
 		return FALSE;
 	}
-	
+
 	gnome_scan_preview_area_set_pixbuf (GNOME_SCAN_PREVIEW_AREA (priv->preview_area),
 										pixbuf,
 										PREVIEW_RES);
@@ -1298,7 +1299,7 @@ gsd_preview_monitor (GnomeScanDialog *gsd)
 }
 
 static void
-	gsd_preview_refresh (GtkButton *button, GnomeScanDialog *gsd)
+gsd_preview_refresh (GtkButton *button, GnomeScanDialog *gsd)
 {
 	static GdkPoint origin = {0, 0};
 	GnomeScanDialogPrivate *priv = GET_PRIVATE (gsd);
@@ -1312,8 +1313,8 @@ static void
 	GTK_WIDGET_SET_VISIBLE(priv->preview_box, FALSE);
 	
 	/* CONFIGURE */
-	
-	/* USE 50dpi RESOLUTION */
+
+	/* USE hard coded RESOLUTION */
 	priv->saved_res = gnome_scan_settings_get (settings, "resolution");
 	/* transform to int or double */
 	pspec = gnome_scan_plugin_params_lookup (GNOME_SCAN_PLUGIN (scanner), "resolution");
@@ -1322,31 +1323,14 @@ static void
 	g_param_value_set_default (pspec, value);
 	g_value_transform (priv->preview_res, value);
 	gnome_scan_settings_set (settings, "resolution", value);
-	gnome_scan_plugin_configure (GNOME_SCAN_PLUGIN (scanner), settings);
 	g_value_unset (value);
-	
-	/* USE manual max PAPER SIZE */
-	priv->saved_paper_size = gnome_scan_settings_get (settings, "paper-size");
-	g_value_init (value, G_VALUE_TYPE (priv->saved_paper_size));
-	g_value_copy (priv->saved_paper_size, value);
-	pspec = gnome_scan_plugin_params_lookup (GNOME_SCAN_PLUGIN (scanner), "paper-size");
-	/* ugly, should rely on papername "maximal" */
-	g_value_set_boxed (value,
-					   GS_PARAM_SPEC_PAPER_SIZE (pspec)->enumeration->next->data);
-	gnome_scan_settings_set (settings, "paper-size", value);
-	g_value_unset (value);
-	
-	priv->saved_origin = gnome_scan_settings_get (settings, "origin");
-	g_value_init (value, G_VALUE_TYPE (priv->saved_origin));
-	g_value_copy (priv->saved_origin, value);
-	g_value_set_pointer (value, &origin);
-	gnome_scan_settings_set (settings, "origin", value);
-	g_value_unset (value);
-	
 	g_free (value);
+	/* asking preview mode */
+	gnome_scan_settings_set_boolean (settings, "preview", TRUE);
 	
 	gnome_scan_job_configure (priv->preview_job);
 	
+	/* TRIGGER */
 	g_timeout_add (42, (GSourceFunc) gsd_preview_monitor, gsd);
 	g_thread_create ((GThreadFunc) gnome_scan_job_run_once,
 					 priv->preview_job, FALSE, &error);
@@ -1355,7 +1339,7 @@ static void
 
 
 static void
-	gsd_preview_cancel_refresh (GtkButton *button, GnomeScanDialog *gsd)
+gsd_preview_cancel_refresh (GtkButton *button, GnomeScanDialog *gsd)
 {
 	GnomeScanDialogPrivate *priv = GET_PRIVATE (gsd);
 	
@@ -1366,7 +1350,7 @@ static void
 
 
 static void
-	gsd_build_preview_ui (GnomeScanDialog *gsd)
+gsd_build_preview_ui (GnomeScanDialog *gsd)
 {
 	GnomeScanDialogPrivate *priv = GET_PRIVATE (gsd);
 	GtkWidget *box, *hbox, *vbox, *alignment, *child, *scrolled;
@@ -1406,6 +1390,7 @@ static void
 	priv->preview_stage = gtk_label_new (NULL);
 	gtk_misc_set_alignment (GTK_MISC (priv->preview_stage), 0., .5);
 	gtk_label_set_markup (GTK_LABEL (priv->preview_stage),
+						  /* translator: initial state before preview begin. */
 						  g_strconcat("<i>", _("Inactive"), "</i>", NULL));
 	gtk_box_pack_start (GTK_BOX (box), priv->preview_stage, FALSE, FALSE, 0);
 	
